#ifndef ANNOTATION_COLOR_H
#define ANNOTATION_COLOR_H

#include <map>
#include <string>

#include <boost/shared_ptr.hpp>

#include "color.hpp"
#include "sequence.hpp"

struct DefaultColorMap;
struct DefaultColorMap
{
  typedef std::map<std::string, DefaultColorMap> color_map_type;
  typedef color_map_type::iterator iterator;
  typedef color_map_type::const_iterator const_iterator;
  DefaultColorMap();
  //! initialize color map with a default color
  DefaultColorMap(const Color &);
  DefaultColorMap(const color_map_type&);
  DefaultColorMap(const DefaultColorMap&);

  Color defaultColor;
  //! color map
  color_map_type cm;
};

class AnnotationColors;
typedef boost::shared_ptr<AnnotationColors> AnnotationColorsRef;

class AnnotationColors
{
public:
  AnnotationColors();
  AnnotationColors(const AnnotationColors &);

  // clear all the contents of our mappers
  void clear();
  //! set default color
  void setColor(Color &);
  //! retreive default color
  Color color();

  //! add default color for a particular type
  void appendTypeColor(const std::string &, const Color &);
  //! get default color for a type
  Color typeColor(const std::string &);
  //! remove a type (and all of its instances)
  void erase(const std::string &);

  //! add default color for a particular type
  void appendInstanceColor(const std::string &type, 
                           const std::string &instance, 
                           const Color &);
  //! get color for a particular type
  Color instanceColor(const std::string &type, 
                      const std::string &instance);
  //! remove an instance of particular type
  void erase(const std::string &type, const std::string& instance);

  //! lookup an annotation color
  Color lookup(const annot &) const;
  Color lookup(const std::string &, const std::string &) const;
private:
  // nested maps, with default?
  DefaultColorMap root_map;
};
#endif
