#include "annotations.hpp"

const std::string Annotations::name_str("name");

Annotations::Annotations()
{
  metadata[name_str] = ""; 
}

Annotations::Annotations(const std::string &n)
{ 
  metadata[name_str] = n; 
}

Annotations::Annotations(const Annotations& a) :
 metadata(a.metadata) 
{
}

Annotations::Annotations(const AnnotationsRef a) :
  metadata(a->metadata)
{
} 

AnnotationsRef Annotations::copy() const
{
  AnnotationsRef a(new Annotations(*this));
  return a;
}

void Annotations::erase(const std::string& key)
{
  if (key == name_str) {
    // we must always have a name
    metadata[key] = "";
  } else {
    metadata.erase(key);
  }
}
  
void Annotations::set(const std::string& key, const std::string &value )
{
  metadata[key] = value;
}

std::string Annotations::get(const std::string& key) const
{
  metadata_map::const_iterator map_i(metadata.find(key));
  if (map_i == metadata.end()) {
    throw annotations_key_error(key + " was not found");
  } else {
    return map_i->second;
  }  
}

bool Annotations::has_key(const std::string& key) const
{
  metadata_map::const_iterator map_i(metadata.find(key));
  return map_i != metadata.end();
}