#include <QDialog>
#include <QFileDialog>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QLabel>
#include <QPixmap>
#include <QPushButton>
#include <QGLWidget>

// Win32 debug includes
//#include <QMessageBox>
//#include <sstream>
//#include <string>
//#include <QString>

#include "ImageSaveDialog.hpp"
#include "ImageScaler.hpp"

#include <iostream>

#include <boost/filesystem/path.hpp>
namespace fs = boost::filesystem;

ImageSaveDialog::ImageSaveDialog(boost::shared_ptr<QDir> default_dir_,
				 ScrollableSequenceBrowser *scrollSeqBrowser, 
				 QWidget *parent)
  : QDialog(parent),
    glwidget(scrollSeqBrowser->browser(), parent),
    default_dir(default_dir_)
{
  setWindowTitle(tr("Mussa Save Image"));

  // Init

  //pixmap = 0;
  imageScaler = new ImageScaler();

  imageScaler->setWidth(static_cast<int>(glwidget.width()));
  imageScaler->setHeight(static_cast<int>(glwidget.height()));
  
  // Save Button
  saveButton = new QPushButton(tr("Save"));
  saveButton->setDefault(true);
  connect(saveButton, SIGNAL(clicked()),
	  this, SLOT(accept()));

  // Cancel Button
  cancelButton = new QPushButton(tr("Cancel"));
  connect(cancelButton, SIGNAL(clicked()),
	  this, SLOT(reject()));

  // Layout
  QHBoxLayout *button_layout = new QHBoxLayout;
  button_layout->addStretch(1);
  button_layout->addWidget(saveButton);
  button_layout->addWidget(cancelButton);
  
  QHBoxLayout *image_centerer = new QHBoxLayout;
  image_centerer->addStretch(1);
  image_centerer->addWidget(imageScaler);
  image_centerer->addStretch(1);
  QVBoxLayout *layout = new QVBoxLayout;
  
  layout->addStretch(1);
  layout->addLayout(image_centerer);
  layout->addStretch(1);
  layout->addLayout(button_layout);
  setLayout(layout);

}

QDir ImageSaveDialog::defaultDir() const
{
  return *default_dir;
}

void ImageSaveDialog::accept()
{
  bool endDialog = true;
  savePixmap(endDialog);
  if (endDialog) {
    done(1);
  }
}

void ImageSaveDialog::reject()
{
  done(0);
}

void ImageSaveDialog::setSize(int width, int height)
{
  imageScaler->setWidth(width);
  imageScaler->setHeight(height);
}

QSize ImageSaveDialog::getOpenGlPixmapSize()
{
  int width = imageScaler->getWidth();
  int height = imageScaler->getHeight();
  
  if (height > 0 && width > 0)
  {
    return QSize(width, height);
  }

  return QSize();
}

QPixmap ImageSaveDialog::renderOpenGlPixmap()
{
  QSize size = getOpenGlPixmapSize();
  if (size.isValid()) 
  {
    double cur_bp_per_pixel = glwidget.zoom();
    float cur_width = glwidget.width();
    float cur_height = glwidget.height();
    
    // Useful debug code (in windows)... hopefully never to be used again. =o)
    //QString msg = "Zoom:   " + QString().setNum(cur_bp_per_pixel) + "\n" + 
    //  "Width:  " + QString().setNum(cur_width) + "\n" +
    //  "Height: " + QString().setNum(cur_height) + "\n";
    //
    //QMessageBox::information(this, "Debug2", 
    //			     msg );
    //

    // When the width requested by the user is doubled, the 
    // bp per pixel is halved in order to scale the image properly.
    double new_bp_per_pixel = ( cur_width / size.width() ) * cur_bp_per_pixel;

    // bp per pixel needs to be scaled based when the width changes
    glwidget.setZoom(new_bp_per_pixel);

    // draw pixmp
    QPixmap pixmap = glwidget.renderPixmap(size.width(), size.height());

    return pixmap;    
  }
  return QPixmap();
}

void ImageSaveDialog::savePixmap(bool &endDialog)
{
  QString filePath;
  filePath = QFileDialog::getSaveFileName(this,
            "Choose a filename to save image",
            default_dir->absolutePath(),
            "Images (*.png *.jpg)");  

  if (filePath.isNull()) {
    // user canceled
    endDialog = false;;
  } else if (filePath.isEmpty()) {
    // somethings wrong
    endDialog = false;
  } else {
    QPixmap pixmap;
    pixmap = renderOpenGlPixmap();
  
    endDialog = true;
    //Save pixelmap to file!
    if (filePath.endsWith(".png"))
      pixmap.save(filePath, "PNG");
    else if (filePath.endsWith(".jpg"))
      pixmap.save(filePath, "JPG");
    else {
      filePath.append(".png");
      pixmap.save(filePath, "PNG");
    }
    fs::path path(filePath.toStdString(), fs::native);
    default_dir->setPath(path.branch_path().native_directory_string().c_str());
  }
}
